/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef libcontrolppH
#define libcontrolppH

#include <iostream>

using namespace std;

//-----------------------------------------------------------------------------

// Display an error message and quit with exit code 1
void FatalError(const string &err);
// Yes/No dialog
bool Approval(const string &msg, const string &positive_answer="y");

//-----------------------------------------------------------------------------
// Add an element to a vector while using a memory reservation

template<typename A, typename B> void PushBack(A &dest, const B &new_element,
        int mem_step=1, const int mem_step_lim=-1){

    if((dest.size() + 1) > dest.capacity()){
        if(mem_step_lim > 0 && mem_step > mem_step_lim){
            mem_step = mem_step_lim;
            }
        dest.reserve(dest.capacity() + mem_step);
        }

    dest.push_back(new_element);
    }

//-----------------------------------------------------------------------------
// Add n-th element of source vector to dest vector

template<typename A> void PushNthBack(A &dest, const A &source, const size_t n,
        int mem_step=1, const int mem_step_lim=-1){

    if(n < source.size()){
        PushBack(dest, source.at(n), mem_step, mem_step_lim);
        }
    }

//-----------------------------------------------------------------------------
// Add all elements of source vector to dest vector

template<typename A> void PushAllBack(A &dest, const A &source,
        int mem_step=-1, const int mem_step_lim=-1){

    if(mem_step < 0){
        mem_step = source.size();
        }

    for(auto &src : source){
        PushBack(dest, src, mem_step, mem_step_lim);
        }
    }

//-----------------------------------------------------------------------------

#endif
