/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef lcontrolpp_mapH
#define lcontrolpp_mapH

#include <vector>
#include <set>
#include <map>

using namespace std;

//-----------------------------------------------------------------------------
// Get sorted vector of keys of a given map

template<typename A, typename B> vector<A> KeysV(const map<A, B> &m,
        const bool reverse=false){

    vector<A> keys;

    for(auto &el : m){
        keys.push_back(el.first);
        }

    if(reverse){
        sort(keys.rbegin(), keys.rend());
        }
    else{
        sort(keys.begin(), keys.end());
        }

    return keys;
    }

//-----------------------------------------------------------------------------
// Get a set of keys of a given map

template<typename A, typename B> set<A> Keys(const map<A, B> &m){

    set<A> keys;

    for(auto &el : m){
        keys.insert(el.first);
        }

    return keys;
    }

//-----------------------------------------------------------------------------
// Get a value of a given map using a specified key, return an empty value
// (constructed with a default constructor) if the key isn't present in the map

template<typename A, typename B> B GetVal(const map<A, B> &m, const A &key){

    const auto it = m.find(key);

    if(it != m.end()){
        return it->second;
        }

    return {};
    }

//-----------------------------------------------------------------------------

#endif
