/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "permissions.h"
#include "permissions_file_perm.h"
#include "permissions_file_acl.h"
#include "text.h"
#include <sys/stat.h>

//-----------------------------------------------------------------------------
// Check that there is a controversy between given modes

bool Controversy(const TFileMode *m1, const TFileMode *m2){

    // Lambda for cheking controversy between modes of different types
    auto perm_and_acl_contr = [] (const TFilePerm *perm, const TFileACL *acl){
        return (
            perm->Mode() != DONT_CHANGE && (
                acl->User()  != "***" ||
                acl->Group() != "***" ||
                acl->Other() != "***")
            );
        };

    if(m1->Type() == TYPE_FPERM && m2->Type() == TYPE_FPERM){
        const TFilePerm *perm1 = (TFilePerm*)m1;
        const TFilePerm *perm2 = (TFilePerm*)m2;
        return perm1->Controversy(*perm2);
        }

    else if(m1->Type() == TYPE_FPERM && m2->Type() == TYPE_FACL){
        const TFilePerm *perm = (TFilePerm*)m1;
        const TFileACL *acl = (TFileACL*)m2;
        return perm_and_acl_contr(perm, acl);
        }

    else if(m1->Type() == TYPE_FACL && m2->Type() == TYPE_FPERM){
        const TFileACL *acl = (TFileACL*)m1;
        const TFilePerm *perm = (TFilePerm*)m2;
        return perm_and_acl_contr(perm, acl);
        }

    else if(m1->Type() == TYPE_FACL && m2->Type() == TYPE_FACL){
        const TFileACL *acl1 = (TFileACL*)m1;
        const TFileACL *acl2 = (TFileACL*)m2;
        return acl1->Controversy(*acl2);
        }

    return true;
    }

//-----------------------------------------------------------------------------
// Spawn an object of a child type and return a pointer to that object

TFileMode* TFileMode::Spawn(const bool store_pointer_only){

    TFilePerm *perm;
    TFileACL *acl;

    switch(this->Type()){

        case TYPE_FPERM:
            if(store_pointer_only){
                return new TFilePerm(p_attrs);
                }
            perm = (TFilePerm*)this;
            return new TFilePerm(*perm);

        case TYPE_FACL:
            if(store_pointer_only){
                return new TFileACL(p_attrs);
                }
            acl = (TFileACL*)this;
            return new TFileACL(*acl);
        }

    return NULL;
    }

//-----------------------------------------------------------------------------
// Clear all data fields

void TFileMode::Clear(void){

    path.clear();

    for(auto &attr : attrs){
        attr.second.clear();
        }

    p_attrs = NULL;
    }

//-----------------------------------------------------------------------------
