/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef lcontrolpp_permissionsH
#define lcontrolpp_permissionsH

#include "permissions_basic.h"
#include "permissions_sectype.h"
#include "filesystem.h"

#define TYPE_FPERM  0
#define TYPE_FACL   1

//-----------------------------------------------------------------------------
// Regular permission mode or ACL of a file

class TFileMode{

    protected:

        string path; // Full file path
        map<string, string> attrs; // Attributes of a file
        const map<string, string> *p_attrs = NULL;

    public:

        // Get ID of a mode type
        virtual int Type(void) const=0;
        // Spawn an object of a child type and return a pointer to that object
        TFileMode* Spawn(const bool store_pointer_only=false);
        // Get/write file path
        const string& Path(void) const{
            return path;
            }
        bool WritePath(const string &val){
            if(!val.empty()){
                path = val;
                return true;
                }
            return false;
            }
        // Get main data fields
        virtual vector<string> GetData(void)=0;
        // Check if file is accessible
        bool Access(void){
            return (access(path.c_str(), F_OK) != -1);
            }
        // Assign specified data field with given value
        virtual bool Assgn(const string &var, const string &val)=0;
        // Revise the mode
        virtual bool Revise(const TSectorType &sec, string &err)=0;
        // Apply the mode and write it's string representation to 'new_perm'
        virtual bool Apply(string &err, string *new_perm=NULL)=0;
        // Check compliance between assigned and currently set mode of the file
        virtual bool CheckCompliance(string &err, string &warn)=0;
        // Copy the mode without any checks
        virtual void CopyMode(const vector<string> &m)=0;
        virtual void CopyMode(const string &m)=0;
        // Check that every data field is not empty
        virtual bool Complete(void) const=0;
        // Check that mode can't have any effect
        virtual bool Neutral(void) const=0;

        // Print params of the mode
        virtual void Print(ostream &out) const=0;
        void Print(void) const{
            Print(cout);
            }

        // Clear all data fields
        virtual void Clear(void);

        TFileMode(void){};
        virtual ~TFileMode(void){};
    };

//-----------------------------------------------------------------------------

// Check that there is a controversy between given modes
bool Controversy(const TFileMode *m1, const TFileMode *m2);

//-----------------------------------------------------------------------------

#endif
