/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef lcontrolpp_permissions_file_permH
#define lcontrolpp_permissions_file_permH

#include "permissions.h"

#define FPERM_PARAMS_N  4
#define FPERM_PATH      0
#define FPERM_OWNER     1
#define FPERM_GROUP     2
#define FPERM_MODE      3

//-----------------------------------------------------------------------------
// Permission for a file

class TFilePerm : public TFileMode{

    protected:

        // Copy data of the permission object
        void CopyFilePerm(const TFilePerm &f);
        // Copy data stored by p_attrs to attrs vector
        void PullData(void);
        // Chown ang chgrp of the file
        bool ChangeOwnerAndGroup(string &err, TFilePerm *new_perm=NULL);
        // Chmod of the file
        bool ChangePermMode(string &err, TFilePerm *new_mode=NULL);

    public:

        // Get the type id
        int Type(void) const{
            return TYPE_FPERM;
            }

        // Fields access
        const string& Owner(void) const;
        const string& Group(void) const;
        const string& Mode(void) const;
        string OwnerMode(void) const;
        string GroupMode(void) const;
        string OtherMode(void) const;

        // Get main data fields
        vector<string> GetData(void);

        // Modifiers
        bool WriteOwner(const string &val){
            attrs["owner"] = val;
            return true;
            }
        bool WriteGroup(const string &val){
            attrs["group"] = val;
            return true;
            }
        bool WriteMode(const string &val){
            if(CheckModeString(val)){
                attrs["mode"] = val;
                return true;
                }
            return false;
            }

        void CopyMode(const vector<string> &m);
        void CopyMode(const string &m);
        bool WriteEverything(const string &p, const string &o, const string &g,
                const string &m);
        bool Assgn(const string &var, const string &val);
        bool Revise(const TSectorType &sec, string &err);
        bool Apply(string &err, string *new_perm=NULL);
        bool CheckCompliance(string &err, string &warn);
        bool Controversy(const TFilePerm &other_perm) const;
        bool Complete(void) const;
        bool Neutral(void) const;
        void Print(ostream &out) const;

        void Clear(void){
            TFileMode::Clear();
            }

        TFilePerm(void){
             attrs["owner"];
             attrs["group"];
             attrs["mode"];
             p_attrs = &attrs;
             }
        TFilePerm(const string &p, const string &o, const string &g,
                const string &m);
        TFilePerm(const map<string, string> *attributes);
        TFilePerm(const TFilePerm &f);
        TFilePerm &operator=(const TFilePerm &f);

        ~TFilePerm(void){}
    };

//-----------------------------------------------------------------------------

// Get current permission of a file
TFilePerm GetPermission(const string &file_path, string &err);

//-----------------------------------------------------------------------------

#endif
