/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "permissions_group_mode.h"

//-----------------------------------------------------------------------------
// Modify specified data field

bool TGroupOfFilesMode::WriteAttr(string &attr, const string &attr_val,
        const bool mode_attr){

    if(attr_val.empty()){
        return false;
        }

    if(mode_attr && !CheckModeString(attr_val)){
        return false;
        }

    attr = attr_val;

    return true;
    }

//-----------------------------------------------------------------------------
// Write excluded paths of a group of files (empty paths are not allowed)

bool TGroupOfFilesMode::WriteExcludedPaths(const vector<string> &epaths){

    bool flag = false;

    for(const auto &epath : epaths){
        if(epath.empty()){
            continue;
            }
        if(!flag){
            excluded_paths.clear();
            flag = true;
            }
        excluded_paths.push_back(epath);
        }

    return flag;
    }

//-----------------------------------------------------------------------------
// Add path to the excluded paths of the group of files

bool TGroupOfFilesMode::AddExcludedPath(const string &epath){

    if(epath.empty()){
        return false;
        }

    excluded_paths.push_back(epath);

    return true;
    }

//-----------------------------------------------------------------------------
// Assign specified data field of the group of files with a given value

bool TGroupOfFilesMode::Assgn(const string &var, const vector<string> &vals){

    if(vals.empty()){
        return false;
        }

    if(var == "mode_for_dirs"){
        return WriteAttr(nested_dirs_mode, vals.front(), true);
        }

    if(var == "base_dir"){
        return WriteBaseDir(vals.front());
        }

    if(var == "excluded_paths"){
        return WriteExcludedPaths(vals);
        }

    return false;
    }

//-----------------------------------------------------------------------------
// Revise the mode

bool TGroupOfFilesMode::Revise(const TSectorType &sec, string &err){

    if(nested_dirs_mode.empty()){
        WriteDirsMode(DONT_CHANGE);
        }

    return true;
    }

//-----------------------------------------------------------------------------
