/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef lcontrolpp_permissions_group_modeH
#define lcontrolpp_permissions_group_modeH

#include "permissions.h"

//-----------------------------------------------------------------------------
// Additional params for a group of files (dir or txt list of file paths)

class TGroupOfFilesMode{

    private:

        string nested_dirs_mode; // Mode for nested dirs
        string base_dir;         // Other paths are relative to the base dir
        vector<string> excluded_paths; // Files excluded from the mode setting

        // Modify specified data field
        bool WriteAttr(string &attr, const string &attr_val,
                const bool mode_attr=false);

    public:

        // Fields access
        const string& DirsMode(void) const{
            return nested_dirs_mode;
            }
        const string& BaseDir(void) const{
            return base_dir;
            }
        const vector<string>& ExcludedPaths(void) const{
            return excluded_paths;
            }
        vector<string> BaseDirExcludedPaths(void) const{
            return RebaseFilePaths(excluded_paths, base_dir);
            }

        // Modifiers
        bool WriteDirsMode(const string &val){
            return WriteAttr(nested_dirs_mode, val, true);
            }
        bool WriteBaseDir(const string &val){
            return WriteAttr(base_dir, FormatFilePath(val), false);
            }
        bool WriteExcludedPaths(const vector<string> &epaths);
        bool AddExcludedPath(const string &epath);
        bool WriteEverything(const string &p, const string &o, const string &g,
                const string &m, const string &md);

        bool Assgn(const string &var, const vector<string> &vals);
        bool Assgn(const string &var, const string &val){
            return Assgn(var, vector<string>(1, val));
            }
        bool Revise(const TSectorType &sec, string &err);
        bool Complete(void) const{
            return !nested_dirs_mode.empty();
            }
        bool Neutral(void) const{
            return nested_dirs_mode == DONT_CHANGE;
            }

        void Clear(void){
            nested_dirs_mode.clear();
            base_dir.clear();
            excluded_paths.clear();
            }
    };

//-----------------------------------------------------------------------------

#endif
