/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef lcontrolpp_permissions_sectypeH
#define lcontrolpp_permissions_sectypeH

#include "libcontrol++.h"

//-----------------------------------------------------------------------------
// Description of a section of a control++ permission mode

class TSectorType{

    private:

        bool valid = false;
        bool file = false;
        bool dir = false;
        bool colorlist = false;
        bool whitelist = false;
        bool blacklist = false;
        bool acl = false;
        bool recursive = false;

    public:

        // Fields access
        bool Valid(void) const{
            return valid;
            }
        bool File(void) const{
            return file;
            }
        bool Dir(void) const{
            return dir;
            }
        bool List(void) const{
            return (colorlist || whitelist || blacklist);
            }
        bool WhiteList(void) const{
            return whitelist;
            }
        bool BlackList(void) const{
            return blacklist;
            }
        bool ACL(void) const{
            return acl;
            }
        bool Recursive(void) const{
            return recursive || whitelist || blacklist;
            }

        // Determine exact type of a section by analysis of it's name
        void FormType(const string &sec_name);

        // Check that section is a 'dir' section
        bool DirSector(const string &sec_name, const string &suffix=""){
            return sec_name == ("dir" + suffix) ||
                sec_name == ("dir_r" + suffix);
            }

        // Check that section is a 'whitelist'/'blacklist' or just a 'list' section
        bool BlackListSector(const string &sec_name, const string &suffix=""){
            return sec_name == ("blacklist" + suffix) ||
                sec_name == ("blacklist_r" + suffix);
            }
        bool WhiteListSector(const string &sec_name, const string &suffix=""){
            return sec_name == ("whitelist" + suffix) ||
                sec_name == ("whitelist_r" + suffix);
            }
        bool ColorListSector(const string &sec_name, const string &suffix=""){
            return sec_name == ("list" + suffix) ||
                sec_name == ("list_r" + suffix);
            }

        // Reset the data
        void Clear(void){
            valid = file = dir = colorlist = whitelist = blacklist
                = acl = recursive = false;
            }
    };

//-----------------------------------------------------------------------------

#endif
