/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef lcontrolpp_stringifyH
#define lcontrolpp_stringifyH

#include <sstream>
#include <map>

using namespace std;

//-----------------------------------------------------------------------------
// Produce a string representation of a given iterable

template<template<typename> typename A, typename B>
        string Stringify(const A<B> &iterable){

    ostringstream oss;

    for(auto el = iterable.begin(); el != iterable.end(); el++){
        if(el != iterable.begin()){
            oss << ", ";
            }
        oss << *el;
        }

    return "{" + oss.str() + "}";
    }

//-----------------------------------------------------------------------------
// Produce a string representation of a given map of iterables

template<typename A, template<typename> typename B, typename C>
        string Stringify(const map <A, B<C>> &m){

    ostringstream oss;

    for(auto kv = m.begin(); kv != m.end(); kv++){
        if(kv != m.begin()){
            oss << ", ";
            }
        oss << kv->first << ": " << Stringify(kv->second);
        }

    return "{" + oss.str() + "}";
    }

//-----------------------------------------------------------------------------

#endif
