/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef lcontrolpp_textH
#define lcontrolpp_textH

#include "libcontrol++.h"
#include <array>
#include <vector>
#include <set>

const set<char> controlpp_spec_symbols = {'\\', '\'', '"', '#'};

//-----------------------------------------------------------------------------

// Check if specified char is tab or space symbol
bool IsTabOrSpace(char c);
// Conver string to uppercase
string ToUpper(string str);
// Conver string to lowercase
string ToLower(string str);
// Mirror single quotes
string MirrorSingleQuotes(const string &line);
// Get position of n-th ocurrance of a substring in a string
size_t FindSubstrNthOccurrence(const size_t n, const string &subs,
    const string &str);
// Search for a next word in a given line and return an index of a last
// character of this word
size_t GetWordFromLine(const string &line, string &word, size_t start=0,
    size_t end=string::npos);
// Trim chars from the left
void LTrim(string &s, const string &chars="");
// Trim chars from the right
void RTrim(string &s, const string &chars="");
// Trim chars from both sides
void Trim(string &s, const string &chars="",
    const bool ltrim=true, const bool rtrim=true);
// Split given string by extracting substrings placed between occurrences
// of a specified delimiter
vector<string> Split(const string &str, const string &delim="");
// Cut comment from a given line
string CutComment(const string &line, const string &comment_designation="#");
// Check that there are specified symbols in a given string
bool CheckSymbols(const string &str,
    const set<char> &symbols=controlpp_spec_symbols);
// Escape symbols of a given set in a given string
string Esc(const string &str,
    const set<char> &esc_symbols=controlpp_spec_symbols);
// Remove an escaping of symbols of a given string
string Unesc(const string &str,
    const set<char> &esc_symbols=controlpp_spec_symbols);
// Check if symbol is escaped
bool Escaped(const string &str, const size_t index,
    const set<char> &esc_symbols=controlpp_spec_symbols);
// Find position of a first unescaped quotation mark in a given string
// starting from the 'pos'
size_t UnescapedQtnMarkPos(const string &str, size_t pos,
    const bool double_quote);

//-----------------------------------------------------------------------------
// Join elements of an iterable by placing copies of a given string in between

template<typename T> string Join(const T &iterable, const string &s){

    string res;

    for(auto i = iterable.begin(); i != iterable.end(); i++){
        if(i != iterable.begin()){
            res += s;
            }
        res += *i;
        }

    return res;
    }

//-----------------------------------------------------------------------------

#endif
