/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef lcontrolpp_tickH
#define lcontrolpp_tickH

#include "libcontrol++.h"
#include <thread>
#include <functional>

//-----------------------------------------------------------------------------
// The TTick constructor can take a function that generates a message string
// using a single unsigned integer parameter; That message then would be
// displayed with a specified time period (second parameter) using a separate
// thread that can be started by calling the "Start" method; The value of the
// unsigned integer that is used to generate the message, can be incremented by
// calling the "Count" method; To stop the process call the "Stop" method; To
// check whether the process had been started, call the "Activated" method.

class TTick{

    private:

        thread t;
        unsigned period;
        unsigned counter = 0;
        bool activated = false;
        bool stop = false;
        function<string(const unsigned)> Msg;

    public:

        void Start(void);
        void Stop(void);
        const bool Activated(void) const{
            return activated;
            }
        void Count(void){
            counter++;
            }

        TTick(function<string(const unsigned&)> print_msg, unsigned tp=3);
        TTick(unsigned tp=3);
        ~TTick(void){};
    };

//-----------------------------------------------------------------------------

#endif
