/*
    control++ : System configuration tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef lcontrolpp_vectorH
#define lcontrolpp_vectorH

#include <string>
#include <vector>
#include <set>

using namespace std;

//-----------------------------------------------------------------------------

// Remove empty strings from a given vector
void VRemEmpty(vector<string> &v);

//-----------------------------------------------------------------------------
// Get vector index of an element with specified value

template<typename A> int VIndex(const vector<A> &v, const A &el){

    if(v.empty()){
        return -1;
        }

    for(size_t i = 0; i < v.size(); i++){
        if(el == v[i]){
            return i;
            }
        }

    return -1;
    }

//-----------------------------------------------------------------------------
// Check that vector contents given value

template<typename A> bool VContents(const vector<A> &v, const A &el){

    return VIndex(v, el) >= 0;
    }

//-----------------------------------------------------------------------------
// Get subvector of a given vector

template<typename A> vector<A> VSub(const vector<A>& v, const int start,
        const int length=-1){

    auto first = (start >= 0 && start <= (int)v.size()) ?
        (v.begin() + start) : v.begin();

    auto last = (length >= 0 && length <= (v.end() - first)) ?
        (first + length) : v.end();

    return vector<A>(first, last);
    }

//-----------------------------------------------------------------------------
// Get elements of vector x that are present in vector y

template<typename A> vector<A> VIntersec(const vector<A> &vx, const vector<A> &vy){

    vector<A> intersec;
    set<A> sy(vy.begin(), vy.end());

    for(auto &x : vx){
        if(sy.find(x) != sy.end()){
            intersec.push_back(x);
            }
        }

    return intersec;
    }

//-----------------------------------------------------------------------------
// Get elements of vector x that are not present in vector y

template<typename A> vector<A> VMinus(const vector<A> &vx, const vector<A> &vy){

    vector<A> diff;
    set<A> sy(vy.begin(), vy.end());

    for(auto &x : vx){
        if(sy.find(x) == sy.end()){
            diff.push_back(x);
            }
        }

    return diff;
    }

//-----------------------------------------------------------------------------

#endif
